<?php

/**
 * @file
 * Functions to support NTT Data theme settings.
 */

use Drupal\Core\Form\FormStateInterface;

/**
 * Implements hook_form_FORM_ID_alter() for system_theme_settings.
 */
function nttdata_theme_form_system_theme_settings_alter(&$form, FormStateInterface $form_state) {
  $form['#validate'][] = 'nttdata_theme_settings_validate';
  $form['#attached']['library'][] = 'nttdata_theme/color-picker';

  $color_config = [
    'colors' => [
      'base_primary_color' => 'Primary Base Color',
      'base_secondary_color' => 'Secondary Color',
      'base_tertiary_color' => 'Tertiary Color',
      'base_quaternary_color' => 'Quaternary Color', 
      'base_quinary_color' => 'Quinary Color', 
    ],
    'schemes' => [
      'default' => [
        'label' => 'Blue Lagoon',
        'colors' => [
          'base_primary_color' => '#070e27', //$midnight-blue
          'base_secondary_color' => '#091232', //$midnight-blue-light
          'base_tertiary_color' => '#1f3ba7', //$navy-blue
          'base_quaternary_color' => '#79a0db', //$light-blue
          'base_quinary_color' => '#cfd8fc', //$cornflower-blue
        ],
      ],
      // Other color schemes...
    ],
  ];

  $form['#attached']['drupalSettings']['nttdata_theme']['colorSchemes'] = $color_config['schemes'];

  $form['nttdata_theme_settings']['nttdata_theme_utilities'] = [
    '#type' => 'fieldset',
    '#title' => t('NTT Data Utilities'),
  ];

  // Define specific fallback colors for each base color including new ones.
  $fallback_colors = [
    'base_primary_color' => '#070e27',  // Fallback for Primary Base Color
    'base_secondary_color' => '#091232', // Fallback for Secondary Color
    'base_tertiary_color' => '#1f3ba7',   // Fallback for Tertiary Color
    'base_quaternary_color' => '#79a0db',  // Fallback for Quaternary Color
    'base_quinary_color' => '#cfd8fc',  // Fallback for Quinary Color
  ];

  foreach ($color_config['colors'] as $key => $title) {
    $default_value = theme_get_setting($key);

    // Use the fallback color if the setting is empty or invalid.
    if (empty($default_value) || !preg_match('/^#[a-fA-F0-9]{6}$/', $default_value)) {
      $default_value = $fallback_colors[$key];  // Use the specific fallback color
    }

    $form['nttdata_theme_settings']['nttdata_theme_utilities']['nttdata_theme_color_scheme'][$key] = [
      '#type' => 'textfield',
      '#maxlength' => 7,
      '#size' => 10,
      '#title' => t($title),
      '#description' => t('Enter color in full hexadecimal format (#abc123).') . '<br/>' . t('Derivatives will be formed from this color.'),
      '#default_value' => $default_value,  // Use the specific fallback color.
      '#attributes' => [
        'pattern' => '^#[a-fA-F0-9]{6}',
      ],
      '#wrapper_attributes' => [
        'data-drupal-selector' => 'nttdata-theme-color-picker',
      ],
    ];
  }

  // Set default values
  $default_font_url = 'https://fonts.googleapis.com/css2?family=Noto+Sans:ital,wght@0,100;0,400;0,700;0,800;1,400;1,700&display=swap';
  $default_font_family = 'Noto Sans, sans-serif';

  // Fetch user-defined values, if any
  $current_font_url = theme_get_setting('theme_font_url') ?: $default_font_url;
  $current_font_family = theme_get_setting('theme_font_family') ?: $default_font_family;

  // Add field for custom font URL
  $form['nttdata_theme_settings']['nttdata_theme_utilities']['theme_font_url'] = [
    '#type' => 'textfield',
    '#title' => t('Custom Font URL'),
    '#description' => t('Provide the URL for a Google Font or another web font. Example: https://fonts.googleapis.com/css2?family=Noto+Sans:wght@400;700&display=swap'),
    '#default_value' => $current_font_url, // Use the defined value or the default
    '#size' => 60,
    '#maxlength' => 256,
  ];

  // Add field for the name of the custom font family
  $form['nttdata_theme_settings']['nttdata_theme_utilities']['theme_font_family'] = [
    '#type' => 'textfield',
    '#title' => t('Custom Font Family Name'),
    '#description' => t('Enter the name of the font family (e.g., "Noto Sans, sans-serif").'),
    '#default_value' => $current_font_family, // Use the defined value or the default
    '#size' => 60,
    '#maxlength' => 256,
    '#attributes' => [
      'pattern' => '^[A-Za-z0-9\s,\';\-\(\)\[\]\"]+$',  // Allows special characters in the font name.
    ],
  ];

}

/**
 * Validation handler for the NTT Data system_theme_settings form.
 */
function nttdata_theme_settings_validate($form, FormStateInterface $form_state) {
  $color_fields = ['base_primary_color', 'base_secondary_color', 'base_tertiary_color', 'base_quaternary_color', 'base_quinary_color'];

  foreach ($color_fields as $field) {
    $color_value = $form_state->getValue($field);
    
    // Only validate if the input is not empty.
    if (!empty($color_value) && !preg_match('/^#[a-fA-F0-9]{6}$/', $color_value)) {
      $form_state->setErrorByName($field, t('The %field value must be a valid 7-character hexadecimal color string.', ['%field' => $field]));
    }
  }

  // URL validation (unchanged).
  $font_url = $form_state->getValue('theme_font_url');
  if (!empty($font_url) && !filter_var($font_url, FILTER_VALIDATE_URL)) {
    $form_state->setErrorByName('theme_font_url', t('The font URL must be a valid URL.'));
  }

  // Validation for the custom font name (unchanged).
  $font_family = $form_state->getValue('theme_font_family');
  if (!empty($font_family) && !preg_match('/^[A-Za-z0-9\s,\';\-\(\)\[\]\"]+$/', $font_family)) {
    $form_state->setErrorByName('theme_font_family', t('The font family name must contain only letters, numbers, commas, spaces, and allowed special characters.'));
  }
}
